'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = convertToPropTypes;

var _util = require('./util');

function convertToPropTypes(node, typesToIdentifiers) {
  (0, _util.$debug)('convertToPropTypes', node);
  var resultPropType = void 0;

  if (node.type === 'ObjectTypeAnnotation') {
    var ret = node.properties.map(function (subnode) {
      var key = subnode.key.name;
      var value = subnode.value;

      // recurse
      value = convertToPropTypes(value, typesToIdentifiers);

      // handles id?: string
      if (value) {
        value.isRequired = !subnode.optional && !value.optional;
      }

      return { key: key, value: value };
    });

    resultPropType = { type: 'shape', properties: ret };
  } else if (node.type === 'FunctionTypeAnnotation') resultPropType = { type: 'func' };else if (node.type === 'AnyTypeAnnotation') resultPropType = { type: 'any' };else if (node.type === 'NumberTypeAnnotation') resultPropType = { type: 'number' };else if (node.type === 'StringTypeAnnotation') resultPropType = { type: 'string' };else if (node.type === 'BooleanTypeAnnotation') resultPropType = { type: 'bool' };else if (node.type === 'VoidTypeAnnotation') resultPropType = { type: 'void' };else if (node.type === 'NullableTypeAnnotation') {
    resultPropType = convertToPropTypes(node.typeAnnotation, typesToIdentifiers);
    resultPropType.optional = true;
  } else if (node.type === 'GenericTypeAnnotation' || node.type === 'ArrayTypeAnnotation') {
    if (node.type === 'ArrayTypeAnnotation' || node.id.name === 'Array') {
      var arrayType;
      if (node.type === 'ArrayTypeAnnotation') {
        arrayType = node.elementType;
      } else {
        arrayType = node.typeParameters.params[0];
      }
      resultPropType = { type: 'arrayOf', of: convertToPropTypes(arrayType, typesToIdentifiers) };
    } else if (node.id && node.id.name && typesToIdentifiers[node.id.name]) {
      resultPropType = { type: 'raw', value: typesToIdentifiers[node.id.name] };
    } else if (node.id.name === 'Object') {
      resultPropType = { type: 'object' };
    } else {
      resultPropType = { type: 'any' };
    }
  } else if (node.type === 'UnionTypeAnnotation') {
    var types = node.types;

    var firstTypeType = types[0].type;

    // e.g. 'hello' | 5
    if (/Literal/.test(firstTypeType)) {
      resultPropType = { type: 'oneOf', options: types.map(function (_ref) {
          var value = _ref.value;
          return value;
        }) };
    }
    // e.g. string | number
    else {
        resultPropType = { type: 'oneOfType', options: types.map(function (node) {
            return convertToPropTypes(node, typesToIdentifiers);
          }) };
      }
  }

  if (resultPropType) {
    return resultPropType;
  } else {
    throw new Error(_util.PLUGIN_NAME + ': Encountered an unknown node in the type definition. Node: ' + JSON.stringify(node));
  }
}