'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = makePropTypesAst;

var _util = require('./util');

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function makePropTypesAst(propTypeData) {
  var rootProperties = propTypeData.properties.map(function (_ref) {
    var key = _ref.key;
    var value = _ref.value;

    return t.objectProperty(t.identifier(key), makePropType(value));
  });
  return t.objectExpression(rootProperties);
};

function makePropType(data) {
  var method = data.type;

  var reactNode = t.callExpression(t.identifier('require'), [(0, _util.makeLiteral)('react')]);
  var node = t.memberExpression(reactNode, t.identifier('PropTypes'));
  var isRequired = true;

  if (method === 'any' || method === 'string' || method === 'number' || method === 'bool' || method === 'object' || method === 'array' || method === 'func') {
    node = t.memberExpression(node, t.identifier(method));
  } else if (method === 'raw') {
    node = t.identifier(data.value);
    isRequired = false;
  } else if (method === 'shape') {
    var shapeObjectProperties = data.properties.map(function (_ref2) {
      var key = _ref2.key;
      var value = _ref2.value;

      return t.objectProperty(t.identifier(key), makePropType(value));
    });
    var shapeObjectLiteral = t.objectExpression(shapeObjectProperties);
    node = t.callExpression(t.memberExpression(node, t.identifier('shape')), [shapeObjectLiteral]);
  } else if (method === 'arrayOf') {
    node = t.callExpression(t.memberExpression(node, t.identifier('arrayOf')), [makePropType(data.of)]);
  } else if (method === 'oneOf') {
    node = t.callExpression(t.memberExpression(node, t.identifier('oneOf')), [t.arrayExpression(data.options.map(_util.makeLiteral))]);
  } else if (method === 'oneOfType') {
    node = t.callExpression(t.memberExpression(node, t.identifier('oneOfType')), [t.arrayExpression(data.options.map(makePropType))]);
  } else if (method === 'void') {
    node = dontSetTemplate().expression;
  } else {
    (0, _util.$debug)('Unknown node ' + JSON.stringify(data, null, 2));
    throw new Error(_util.PLUGIN_NAME + ' processing error: This is an internal error that should never happen. ' + ('Report it immediately with the source file and babel config. Data: ' + data));
  }

  if (isRequired && data.isRequired && method !== 'void') {
    node = t.memberExpression(node, t.identifier('isRequired'));
  }

  return node;
}

var dontSetTemplate = (0, _babelTemplate2.default)('\n(props, propName, componentName) => {\n  if(props[propName] != null) return new Error(`Invalid prop \\`${propName}\\` of value \\`${value}\\` passed to \\`${componentName}\\`. Expected undefined or null.`);\n}\n');